/**
  File name ubitx_Kenwood_CAT.cpp
   CAT Support

   This module handles serial communications with external program.
   Decode and executes Kenwood TS-590S protocol plus
   Special uBITX remote commands

*/
#include <Arduino.h>
#include "ubitx.h"
#include "EEPROM.h"
#include <TimeLib.h>
#include <ili9341_t3n_font_Arial.h>
#include <ili9341_t3n_font_ArialBold.h>
#include <ILI9341_t3n.h>
#include <SPIN.h>
#include <SPI.h>

extern ILI9341_t3n tft;

extern void tftPrint(int x, int y, char* Text, ILI9341_t3_font_t, int fontColor, bool textframed, int frameColor );

extern void Process_Memory_Tables();
extern void Display_SPL(void);
extern void Display_SPL_ON(void);
extern void updateDisplay(void);
extern int  btnDown(void);
extern void MenuExit(int btn);
extern void setFrequency(unsigned long xf);
extern void stopTx(void);
extern void startTx(byte txMode);
extern void menuSplitToggle( void );
extern void menuVfoToggle(void );
extern void Display_RIT(void);
extern void Display_RIT_ON(void);
extern void SetIambicA( void );
extern void SetIambicB( void );
extern void menuPaddleSwapToggle( void );
extern void cwKeydown();
extern void cwKeyUp();
extern void digitalClockDisplay();
extern void sendmsg(char *str);
extern void Display_Voltage( void );
extern void Print_Menu_Line(void);
extern bool Put_SDEEPROM( int index, long Value );
extern long Get_SDEEPROM( int index );
extern bool CAT_Volts_Active;
extern void Display_Mem_Selection(void);
extern void Display_Mode( void);
extern void Set_Band(void);
extern int Frequency_is_Band(unsigned long frequency);

extern int Memory_Table_Select;
extern int Mem_Number;

extern void Display_PBT_OFF(void);
extern void Display_PBT_Position(void);
extern long PBT_Value;

extern unsigned long frequency;
extern volatile int ubitx_mode;
extern bool swlUSB;
extern unsigned long vfoA, vfoB;
extern char vfoActive;
extern volatile bool keyDown;
extern unsigned long sideTone;
extern char splitOn;
extern char ritOn;
extern int wpm;
extern unsigned long Freq_Incr[6];
extern int Freq_Index;
extern long ritOffset;
extern char Version_Text[21];
extern char Version_Date[21];
extern bool Volts_Active;
extern volatile unsigned long Ubitx_Voltage;
extern unsigned volatile char IAMBIC;
extern unsigned volatile char PDLSWAP;
extern int wpm;
extern int cwSpeed;
extern unsigned long Volt_Divider;
extern char Time_Buffer[12];
extern volatile int SmeterReadValue, LastSmeter, SMeter_Value, smeterInterval;
extern int Current_Band;

extern int VFOA_BAND;
extern int VFOB_BAND;

/////////////////////////////////////////////////////////////////

bool CAT_Tune = false;
bool Cat_Lock = false;
bool CAT_S_Meter_Active;

volatile static byte CAT_BUFF[64];
volatile static int Cat_Buffer_Index;
volatile bool TX_In_Progress = false;

int ltable, lchannel, lmode;

///////////////////////////////////////////////////////////////////////////////

void Display_RCP(void) {
  tft.drawRect(164, 2, 23, 42,  ILI9341_WHITE);
  tft.fillRect(165, 3, 21, 40,  ILI9341_RED);
  tft.setFont(Arial_8);
  tft.setTextColor(ILI9341_WHITE);
  tft.setCursor(168, 5);
  tft.print("R");
  tft.setCursor(172, 18);
  tft.print("C");
  tft.setCursor(176, 31);
  tft.print("P");
}


void Init_Cat( long baud )
{
  Serial.begin( baud );
  Serial.flush();

  Cat_Buffer_Index = 0;
  CAT_Tune = false;
  Cat_Lock = false;
}


//////////////////////////////////////////////////////////////////////
void Process_Command( void ) {
  char Response_Buffer[64];
  char Freq_Text[12];
  int i = 0, j, k, Channel_Mode ;
  char * temp;
  unsigned long Long_Temp;
  long rx_freq;

  //    tft.fillRect(20, 220, 290, 24, ILI9341_BLACK);
  //    tftPrint(20,
  //             223,
  //             (char*)CAT_BUFF,
  //             Arial_12,
  //             ILI9341_WHITE,
  //             0,
  //             ILI9341_WHITE );



  if ( strncmp((char*)CAT_BUFF, "u", 1) == 0) {
    //////////////////////////////////////////////////////////////    uBITX SPECIFIC CAT COMMANDS  ///////////////////////

    if ( strncmp((char*)CAT_BUFF, "uBD;", 4 ) == 0) {
      Current_Band = Current_Band - 1;
      if ( Current_Band < 0 )Current_Band = 0;
      Set_Band();
      sprintf(Response_Buffer, "uBX%1d;", Current_Band);
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uBU;", 4 ) == 0) {
      Current_Band = Current_Band + 1;
      if ( Current_Band > 7 )Current_Band = 7;
      Set_Band();
      sprintf(Response_Buffer, "uBX%1d;", Current_Band);
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uBX", 3 ) == 0) {
      i = atoi((char*) &CAT_BUFF[3]);
      if ( (i >= 0) && (i < 8)) {
        Current_Band = i;
        Set_Band();
        sprintf(Response_Buffer, "uBX%1d;", Current_Band);
        Serial.write(Response_Buffer);
        delay(10);
      }
    } else if ( strncmp((char*)CAT_BUFF, "uCV;", 4) == 0) {
      strcpy(Response_Buffer, "uCV");
      if ( vfoActive == VFO_A ) strcat(Response_Buffer, "0;");
      else strcat(Response_Buffer, "1;");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "uLA;", 4) == 0) {
      ultoa(vfoA, Freq_Text, DEC);
      strcpy(Response_Buffer, "uLA");
      for (i = 0; i < signed(11 - strlen(Freq_Text)); i++)strcat(Response_Buffer, "0");
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);
      delay(10);


    } else if ( strncmp((char*)CAT_BUFF, "uLA", 3) == 0) {
      strncpy(Freq_Text, (char*)&CAT_BUFF[3], 11);
      rx_freq = strtol(Freq_Text, NULL, 10);
      if ( rx_freq >  400000l)
        if ( rx_freq < 30000000l) {
          vfoA = rx_freq;
          if ( vfoActive == VFO_B ) {
            frequency = vfoA;
            setFrequency(frequency);
            vfoActive = VFO_A;
            menuVfoToggle();
            updateDisplay();
          }
        }

    } else if ( strncmp((char*)CAT_BUFF, "uLB;", 4) == 0) {
      ultoa(vfoB, Freq_Text, DEC);
      strcpy(Response_Buffer, "uLB");
      for (i = 0; i < signed(11 - strlen(Freq_Text)); i++)strcat(Response_Buffer, "0");
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uLB", 3) == 0) {
      strncpy(Freq_Text, (char*)&CAT_BUFF[3], 11);
      rx_freq = strtol(Freq_Text, NULL, 10);
      if ( rx_freq >  400000l)
        if ( rx_freq < 30000000l) {
          vfoB = rx_freq;
          if ( vfoActive == VFO_A ) {
            frequency = vfoB;
            setFrequency(frequency);
            vfoActive = VFO_A;
            menuVfoToggle();
            updateDisplay();
          }
        }

    } else if ( strncmp((char*)CAT_BUFF, "uDT", 3) == 0) {
      if ( CAT_BUFF[3] == '0') i = 0;
      else if ( CAT_BUFF[3] == '1') i = 1;
      else if ( CAT_BUFF[3] == '2') i = 2;
      else if ( CAT_BUFF[3] == '3') i = 3;
      j = strlen((char*)CAT_BUFF) - 1;
      if ( j > 20 )j = 20;
      CAT_BUFF[j] = 0;
      if ( i > 3 ) i = 3;
      strcpy(Response_Buffer, (char*)&CAT_BUFF[4]);
      // printLine(i, Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "uFL", 3) == 0) {
      Cat_Lock = atoi((char*) &CAT_BUFF[3]);
      if ( Cat_Lock == 0 ) {
        Put_SDEEPROM( VFO_A, vfoA );
        Put_SDEEPROM( VFO_B, vfoB );
        tft.fillRect(164, 2, 23, 42,  ILI9341_BLACK);
        LastSmeter = 0;
        updateDisplay();
        Display_Mem_Selection();
      } else {
        strcpy(Response_Buffer, "uFL;");
        Serial.write(Response_Buffer);
        sprintf(Response_Buffer, "uBX%1d;", Current_Band);
        Serial.write(Response_Buffer);
        strcpy(Response_Buffer, "uMD");
        if ( ubitx_mode == MODE_USB )strcat(Response_Buffer, "0");
        else if ( ubitx_mode == MODE_LSB )strcat(Response_Buffer, "1");
        else if ( ubitx_mode == MODE_CW )strcat(Response_Buffer, "2");
        else if ( ubitx_mode == MODE_CWR )strcat(Response_Buffer, "3");
        else if ( ubitx_mode == MODE_SWU )strcat(Response_Buffer, "4");
        else if ( ubitx_mode == MODE_SWL )strcat(Response_Buffer, "5");
        else if ( ubitx_mode == MODE_CWP )strcat(Response_Buffer, "6");
        strcat(Response_Buffer, ";");
        Serial.write(Response_Buffer);
        delay(10);
        Display_RCP();
      }


    } else if ( strncmp((char*)CAT_BUFF, "uKIA;", 5) == 0) {
      SetIambicA();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uKIB;", 5) == 0) {
      SetIambicB();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uKPN;", 5) == 0) {
      PDLSWAP = 0x08;
      menuPaddleSwapToggle();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uKPR;", 5) == 0) {
      PDLSWAP = 0;
      menuPaddleSwapToggle();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uKS;", 4) == 0) {
      strcpy(Response_Buffer, "uKS");
      itoa(wpm, Freq_Text, 10);
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "uKS", 3) == 0) {
      wpm = strtol((char*)&CAT_BUFF[3], NULL, 10);
      cwSpeed = 600 / wpm;
      Display_Mode();
      //  itoa(wpm, Response_Buffer , 10);
      //  tft.setTextColor(ILI9341_WHITE);
      //  tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
      //  tft.setCursor(24, 48 );
      //  tft.setFont(Arial_18);
      //  tft.println(Response_Buffer);
      Put_SDEEPROM( CW_SPEED, wpm );

    } else if ( strncmp((char*)CAT_BUFF, "uMD;", 4) == 0) {
      strcpy(Response_Buffer, "uMD");
      if ( ubitx_mode == MODE_USB )strcat(Response_Buffer, "0");
      else if ( ubitx_mode == MODE_LSB )strcat(Response_Buffer, "1");
      else if ( ubitx_mode == MODE_CW )strcat(Response_Buffer, "2");
      else if ( ubitx_mode == MODE_CWR )strcat(Response_Buffer, "3");
      else if ( ubitx_mode == MODE_SWU )strcat(Response_Buffer, "4");
      else if ( ubitx_mode == MODE_SWL )strcat(Response_Buffer, "5");
      else if ( ubitx_mode == MODE_CWP )strcat(Response_Buffer, "6");
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "uMD", 3) == 0) {
      if ( CAT_BUFF[3] == '0' )ubitx_mode = MODE_USB;
      else if ( CAT_BUFF[3] == '1' ) ubitx_mode = MODE_LSB;
      else if ( CAT_BUFF[3] == '2' ) ubitx_mode = MODE_CW;
      else if ( CAT_BUFF[3] == '3' ) ubitx_mode = MODE_CWR;
      else if ( CAT_BUFF[3] == '4' ) ubitx_mode = MODE_SWU;
      else if ( CAT_BUFF[3] == '5' ) ubitx_mode = MODE_SWL;
      else if ( CAT_BUFF[3] == '6' ) ubitx_mode = MODE_CWP;
      Put_SDEEPROM( MODE, ubitx_mode );
      updateDisplay();
      Display_Mode();


    } else if ( strncmp((char*)CAT_BUFF, "uPB", 3) == 0) {
      if ( CAT_BUFF[3] == '0' )Display_PBT_OFF();
      else {
        PBT_Value = 0;
        Display_PBT_Position();
      }



    } else if ( strncmp((char*)CAT_BUFF, "uPV", 3) == 0) {
      PBT_Value = atol((char*)&CAT_BUFF[3]);
      Display_PBT_Position();


    } else if ( strncmp((char*)CAT_BUFF, "uTL", 3) == 0) {
      // uTLnmffffffff;
      strncpy(Freq_Text, (char*)&CAT_BUFF[3], 1);
      Memory_Table_Select = atoi(Freq_Text);
      strncpy(Freq_Text, (char*)&CAT_BUFF[4], 1);
      Mem_Number = atoi(Freq_Text); // get table
      strncpy(Freq_Text, (char*)&CAT_BUFF[5], 1);
      ubitx_mode = atoi(Freq_Text);
      Display_Mem_Selection();
      strncpy(Freq_Text, (char*)&CAT_BUFF[6], 11);
      rx_freq = strtol(Freq_Text, NULL, 10);
      if ( rx_freq >  400000l)
        if ( rx_freq < 30000000l) {
          if ( vfoActive == VFO_A ) {
            vfoA = rx_freq;
            frequency = vfoA;
          } else {
            vfoB = rx_freq;
            frequency = vfoB;
          }
          Put_SDEEPROM( MODE, ubitx_mode );
          setFrequency(frequency);
          Display_Mode();
          updateDisplay();
        }

    } else if ( strncmp((char*)CAT_BUFF, "uTD", 3) == 0) {
      //uTDtnmffffffff;
      //  strncpy(Freq_Text, (char*)&CAT_BUFF[3], 11);
      //  Memory_Table_Select = strtol(Freq_Text, NULL, 10);

      strncpy(Freq_Text, (char*)&CAT_BUFF[4], 1);
      lchannel = atoi(Freq_Text); // get table
      Mem_Number = lchannel;
      Display_Mem_Selection();
      strncpy(Freq_Text, (char*)&CAT_BUFF[5], 1);
      lmode = atoi(Freq_Text); // get table
      strncpy(Freq_Text, (char*)&CAT_BUFF[6], 8);
      rx_freq = strtol(Freq_Text, NULL, 10);
      j = ( ltable + 1 ) * 100;
      Put_SDEEPROM( (j + 40 + (lchannel * 4)), lmode );
      delay(10);
      Put_SDEEPROM( (j + (lchannel * 4)), rx_freq );

    } else if ( strncmp((char*)CAT_BUFF, "uMT", 3) == 0) {
      k = atoi((char*) &CAT_BUFF[3]);
      Memory_Table_Select = k; // get table
      Process_Memory_Tables();

    } else if ( strncmp((char*)CAT_BUFF, "uTU", 3) == 0) {
      k = atoi((char*) &CAT_BUFF[3]);
      j = ( k + 1 ) * 100;
      for (i = 0; i < 10; i++) {
        rx_freq = Get_SDEEPROM( j + (i * 4) );
        Channel_Mode = Get_SDEEPROM( j + 40 + (i * 4) );
        sprintf(Response_Buffer, "uTU%1d%1d%1d%08d;", k, i, Channel_Mode, (int)rx_freq);
        Response_Buffer[strlen(Response_Buffer) + 1] = 0;
        Serial.print(Response_Buffer);
        delay(10);
      }

    } else if ( strncmp((char*)CAT_BUFF, "uRE;", 4) == 0) {
      Display_RIT();
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uRS;", 4) == 0) {
      Display_RIT_ON();
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uRO", 3) == 0) {
      ritOffset = atol((char*)&CAT_BUFF[3]);
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uSA;", 4) == 0) {
      frequency = vfoA;
      Display_RIT();
      if ( vfoActive == VFO_B ) {
        frequency = vfoA;
        setFrequency(frequency);
        vfoActive = VFO_A;
        menuVfoToggle();
        menuVfoToggle();
      }

      updateDisplay();
      sprintf(Response_Buffer, "uBX%1d;", Frequency_is_Band(frequency));
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uSB;", 4) == 0) {
      frequency = vfoB;
      Display_RIT();
      if ( vfoActive == VFO_A ) {
        frequency = vfoB;
        setFrequency(frequency);
        vfoActive = VFO_B;
        menuVfoToggle();
        menuVfoToggle();
      }

      updateDisplay();
      sprintf(Response_Buffer, "uBX%1d;", Frequency_is_Band(frequency));
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uSC;", 4) == 0) {
      strcpy(Response_Buffer, "uSC");
      for (i = 0; i <= 40; i = i + 4) {
        Long_Temp = Get_SDEEPROM( i );
        sprintf(Response_Buffer, "uSC%02d %08X - %08d;\n", i, int(Long_Temp), int(Long_Temp));
        Serial.write(Response_Buffer);
      }
      for (i = 46; i <= 62; i = i + 4) {
        Long_Temp = Get_SDEEPROM( i );
        sprintf(Response_Buffer, "uSC%02d %08X - %08d;\n", i, int(Long_Temp), int(Long_Temp));
        Serial.write(Response_Buffer);
      }

    } else if ( strncmp((char*)CAT_BUFF, "uSM", 3) == 0) {
      sprintf(Response_Buffer, "%s", &CAT_BUFF[3]);
      Response_Buffer[strlen(Response_Buffer) - 1] = 0;
      sendmsg(Response_Buffer);
      strcpy(Response_Buffer, "uSM;");
      Serial.write(Response_Buffer);
      delay(10);

    } else if ( strncmp((char*)CAT_BUFF, "uMS;", 4) == 0) {
      CAT_S_Meter_Active = true;

    } else if ( strncmp((char*)CAT_BUFF, "uME;", 4) == 0) {
      CAT_S_Meter_Active = false;

    } else if ( strncmp((char*)CAT_BUFF, "uSE;", 4) == 0) {
      Display_SPL();
      menuVfoToggle();
      updateDisplay();
      menuVfoToggle();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uSS;", 4) == 0) {
      Display_SPL_ON();
      vfoB = vfoA;
      menuVfoToggle();
      updateDisplay();
      menuVfoToggle();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uST", 3) == 0) {
      time_t pctime = atol((char*)&CAT_BUFF[3]);
      setTime(pctime);
      Teensy3Clock.set(now());
      digitalClockDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "uKT;", 4) == 0) {
      strcpy(Response_Buffer, "uKT");
      itoa(sideTone, Freq_Text, 10);
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "uKT", 3) == 0) {
      sideTone = strtol((char*)&CAT_BUFF[3], &temp, 10);
      //save the setting
      Put_SDEEPROM( CW_SIDETONE, sideTone );

    } else if ( strncmp((char*)CAT_BUFF, "uVE;", 4) == 0) {
      CAT_Volts_Active = false;

    } else if ( strncmp((char*)CAT_BUFF, "uVS;", 4) == 0) {
      CAT_Volts_Active = true;

    } else if ( strncmp((char*)CAT_BUFF, "uXS;", 4) == 0) {
      TX_In_Progress = true;
      startTx(TX_SSB);

    } else if ( strncmp((char*)CAT_BUFF, "uXE;", 4) == 0) {
      TX_In_Progress = false;
      stopTx();

    } else if ( strncmp((char*)CAT_BUFF, "uTS;", 4) == 0) {
      TX_In_Progress = true;
      startTx(TX_CW);
      cwKeydown();

    } else if ( strncmp((char*)CAT_BUFF, "uTE;", 4) == 0) {
      cwKeyUp();
      TX_In_Progress = false;
      stopTx();
    }

  } else {
    //////////////////////////////  Kensowwd TS-590S  CAT subset

    if ( strncmp((char*)CAT_BUFF, "FA;", 3) == 0) {
      ultoa(vfoA, Freq_Text, DEC);
      strcpy(Response_Buffer, "FA");
      for (i = 0; i < signed(11 - strlen(Freq_Text)); i++)strcat(Response_Buffer, "0");
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "FA", 2) == 0) {
      strncpy(Freq_Text, (char*)&CAT_BUFF[2], 11);
      vfoA = strtol(Freq_Text, NULL, 10);
      frequency = strtol(Freq_Text, NULL, 10);
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "FB;", 3) == 0) {
      ultoa(vfoB, Freq_Text, DEC);
      strcpy(Response_Buffer, "FB");
      for (i = 0; i < signed(11 - strlen(Freq_Text)); i++)strcat(Response_Buffer, "0");
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);


    } else if ( strncmp((char*)CAT_BUFF, "FB", 2) == 0) {
      strncpy(Freq_Text, (char*)&CAT_BUFF[2], 11);
      vfoB = strtol(Freq_Text, NULL, 10);
      frequency = strtol(Freq_Text, NULL, 10);
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "FR;", 3) == 0) {
      strcpy(Response_Buffer, "FR");
      if ( vfoActive == VFO_A )strcat(Response_Buffer, "0");
      else  strcat(Response_Buffer, "1");
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "FR", 2) == 0) {
      if ( CAT_BUFF[2] == '0' ) vfoActive = VFO_B;
      else vfoActive = VFO_A;
      menuVfoToggle();

    } else if ( strncmp((char*)CAT_BUFF, "FT;", 3) == 0) {
      strcpy(Response_Buffer, "FT");
      if ( splitOn )strcat(Response_Buffer, "1");
      else  strcat(Response_Buffer, "0");
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "FT", 2) == 0) {
      if ( CAT_BUFF[2] == '0' ) splitOn = true;
      else splitOn = false;
      menuSplitToggle();
      menuVfoToggle();
      updateDisplay();
      menuVfoToggle();
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "FV;", 3) == 0) {
      strcpy(Response_Buffer, "FV1.00;");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "KS;", 3) == 0) {
      strcpy(Response_Buffer, "KS");
      itoa(wpm, Freq_Text, 10);
      strcat(Response_Buffer, Freq_Text);
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "KS", 2) == 0) {
      wpm = strtol((char*)&CAT_BUFF[2], &temp, 10);
      itoa(wpm, Response_Buffer , 10);
      tft.setTextColor(ILI9341_WHITE);
      tft.fillRect(24, 48, 30, 18, ILI9341_BLACK);
      tft.setCursor(24, 48 );
      tft.println(Response_Buffer);
      Put_SDEEPROM( CW_SIDETONE, sideTone );


    } else if ( strncmp((char*)CAT_BUFF, "AI;", 3) == 0) {
      strcpy(Response_Buffer, "AI0;");
      Serial.write(Response_Buffer);


    } else if ( strncmp((char*)CAT_BUFF, "ID;", 3) == 0) {
      strcpy(Response_Buffer, "ID021;");
      Serial.write(Response_Buffer);


    } else if ( strncmp((char*)CAT_BUFF, "IF;", 3) == 0) {
      if ( vfoActive == VFO_A )i = 0; else i = 1;
      sprintf(Response_Buffer, "IF%011d     %05d%1d0 000%1d%1d0%1d0000;", (int)frequency, (int)ritOffset, ritOn, ubitx_mode, i, splitOn);
      Serial.write(Response_Buffer);


    } else if ( strncmp((char*)CAT_BUFF, "MD;", 3) == 0) {
      strcpy(Response_Buffer, "MD");
      if ( ubitx_mode == MODE_CW )strcat(Response_Buffer, "3");
      else if ( ubitx_mode == MODE_USB )strcat(Response_Buffer, "2");
      else if ( ubitx_mode == MODE_LSB )strcat(Response_Buffer, "1");
      else strcat(Response_Buffer, "7");
      strcat(Response_Buffer, ";");
      Serial.println(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "MD", 2) == 0) {
      if ( CAT_BUFF[2] == '1' )ubitx_mode = MODE_LSB;
      else if ( CAT_BUFF[2] == '2' ) ubitx_mode = MODE_USB;
      else if ( CAT_BUFF[2] == '3' ) ubitx_mode = MODE_CW;
      else if ( CAT_BUFF[2] == '7' ) ubitx_mode = MODE_CWR;
      i = ubitx_mode;
      Put_SDEEPROM( MODE, i );
      setFrequency(frequency);
      updateDisplay();

    } else if ( strncmp((char*)CAT_BUFF, "RT;", 3) == 0) {
      strcpy(Response_Buffer, "RT");
      if ( ritOn )strcat(Response_Buffer, "1");
      else  strcat(Response_Buffer, "0");
      strcat(Response_Buffer, ";");
      Serial.write(Response_Buffer);

    } else if ( strncmp((char*)CAT_BUFF, "RT", 2) == 0) {
      if ( CAT_BUFF[2] == '0' ) Display_RIT();
      else Display_RIT_ON();
      updateDisplay();


    } else if ( strncmp((char*)CAT_BUFF, "RX;", 3) == 0) {
      TX_In_Progress = false;
      stopTx();

    } else if ( strncmp((char*)CAT_BUFF, "TX0;", 4) == 0) {
      if ( frequency >= 10000000 )ubitx_mode = MODE_USB;
      else ubitx_mode = MODE_LSB;
      setFrequency(frequency);
      updateDisplay();
      TX_In_Progress = true;

      startTx(TX_SSB);

    } else if ( strncmp((char*)CAT_BUFF, "VV;", 3) == 0) {
      if ( vfoActive == VFO_A )vfoB = frequency;
      else {
        vfoB = vfoA;
        frequency = vfoA;
      }
      updateDisplay();

    }
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////

void  Process_CAT_Command( void ) {

  // wait for command
  while ( Serial.available() > 0 ) {
    CAT_BUFF[Cat_Buffer_Index] = Serial.read();
    if ( CAT_BUFF[Cat_Buffer_Index ] == ';') {
      CAT_BUFF[Cat_Buffer_Index + 1 ] = 0;
      Process_Command();
      Cat_Buffer_Index = 0;
    } else {
      Cat_Buffer_Index = Cat_Buffer_Index + 1;
    }
  }
}





